'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var mock = require('@midwayjs/mock');
var hooksCore = require('@midwayjs/hooks-core');
var path = require('path');
var fsExtra = require('fs-extra');

async function createAppImplementation(baseDir, isFunction) {
  const root = hooksCore.getProjectRoot(baseDir || global.testPath);
  const config = hooksCore.getConfig(root);
  const cwd = process.cwd();
  process.chdir(root);
  const app = await mock.createApp(root, {
    baseDir: path.join(root, config.source)
  }, isFunction && '@midwayjs/serverless-app');
  process.chdir(cwd);
  return new HooksApplication(app);
}
async function createApp(baseDir) {
  return createAppImplementation(baseDir, false);
}
async function createFunctionApp(baseDir) {
  return createAppImplementation(baseDir, true);
}
class HooksApplication {
  constructor(app) {
    this.app = void 0;
    this.app = app;
  } // TODO Allow pass user define context


  async runFunction(fn, ...args) {
    const response = await this.request(fn, ...args);

    if (response.type === 'application/json') {
      return response.body;
    }

    return response.text;
  }

  request(fn, ...args) {
    const supertest = mock.createHttpRequest(this.app);

    if (fn._param.method === 'GET') {
      return supertest.get(fn._param.url);
    }

    return supertest.post(fn._param.url).send({
      args
    }).set('Content-Type', 'application/json');
  }

  async close() {
    var _this$app;

    await mock.close(this.app);

    if (process.env.EGG_HOME) {
      await fsExtra.remove(path.join(process.env.EGG_HOME, 'logs'));
    }

    const appDir = (_this$app = this.app) === null || _this$app === void 0 ? void 0 : _this$app.getAppDir();

    if (appDir) {
      await fsExtra.remove(path.join(appDir, 'logs'));
      await fsExtra.remove(path.join(appDir, 'run'));
    }
  }

}

exports.HooksApplication = HooksApplication;
exports.createApp = createApp;
exports.createAppImplementation = createAppImplementation;
exports.createFunctionApp = createFunctionApp;
