import { createApp as createApp$1, createHttpRequest, close } from '@midwayjs/mock';
import { getProjectRoot, getConfig } from '@midwayjs/hooks-core';
import { join } from 'path';
import { remove } from 'fs-extra';

async function createAppImplementation(baseDir, isFunction) {
  const root = getProjectRoot(baseDir || global.testPath);
  const config = getConfig(root);
  const cwd = process.cwd();
  process.chdir(root);
  const app = await createApp$1(root, {
    baseDir: join(root, config.source)
  }, isFunction && '@midwayjs/serverless-app');
  process.chdir(cwd);
  return new HooksApplication(app);
}
async function createApp(baseDir) {
  return createAppImplementation(baseDir, false);
}
async function createFunctionApp(baseDir) {
  return createAppImplementation(baseDir, true);
}
class HooksApplication {
  constructor(app) {
    this.app = void 0;
    this.app = app;
  } // TODO Allow pass user define context


  async runFunction(fn, ...args) {
    const response = await this.request(fn, ...args);

    if (response.type === 'application/json') {
      return response.body;
    }

    return response.text;
  }

  request(fn, ...args) {
    const supertest = createHttpRequest(this.app);

    if (fn._param.method === 'GET') {
      return supertest.get(fn._param.url);
    }

    return supertest.post(fn._param.url).send({
      args
    }).set('Content-Type', 'application/json');
  }

  async close() {
    var _this$app;

    await close(this.app);

    if (process.env.EGG_HOME) {
      await remove(join(process.env.EGG_HOME, 'logs'));
    }

    const appDir = (_this$app = this.app) === null || _this$app === void 0 ? void 0 : _this$app.getAppDir();

    if (appDir) {
      await remove(join(appDir, 'logs'));
      await remove(join(appDir, 'run'));
    }
  }

}

export { HooksApplication, createApp, createAppImplementation, createFunctionApp };
